@@ -1,12 +1,15 @@
 from __future__ import annotations
-import inspect
+
 import copy
 from typing import Iterator
 from uuid import uuid4
+
 from anytree import AnyNode
+
 from . import constants, filehandling
 from .external_software import excel
 
+
 # Add child to Parent leads to reverse
 
 class IterRegistry(type):
@@ -30,7 +33,6 @@ class Project(object):
         self.seperator_status = True
         self.seperator = ","
 
-
     @property
     def changed(self) -> bool:
         def check_data():
@@ -104,11 +106,11 @@ class Project(object):
         self.seperator_status = True
         self.seperator = ","
 
-    def open(self, path:str) -> None:
+    def open(self, path: str) -> None:
         filehandling.read_xml(self, path)
 
-    def import_excel(self, path:str,ws_name:str) -> None:
-        excel.open_file(path,ws_name)
+    def import_excel(self, path: str, ws_name: str) -> None:
+        excel.open_file(path, ws_name)
 
     @property
     def objects(self) -> Iterator[Object]:
@@ -127,9 +129,10 @@ class Project(object):
             create_childen(n)
         return base
 
+
 class Hirarchy(object, metaclass=IterRegistry):
 
-    def __init__(self, name: str) -> None:
+    def __init__(self, name: str, description: str | None = None, optional: bool | None = None) -> None:
 
         self._parent = None
         self._children = set()
@@ -139,6 +142,36 @@ class Hirarchy(object, metaclass=IterRegistry):
             constants.SHARED_PARAMETERS: True,
             constants.IFC_MAPPING: True
         }
+        self._description = ""
+        if description is not None:
+            self.description = description
+
+        self._optional = False
+        if optional is not None:
+            self._optional = optional
+
+    @property
+    def optional_wo_hirarchy(self) ->bool:
+        return self._optional
+
+    @property
+    def optional(self) -> bool:
+        if self.parent is not None:
+            if self.parent.optional:
+                return True
+        return self._optional
+
+    @optional.setter
+    def optional(self, value: bool) -> None:
+        self._optional = value
+
+    @property
+    def description(self):
+        return self._description
+
+    @description.setter
+    def description(self, value):
+        self._description = value
 
     @property
     def mapping_dict(self) -> dict[str, bool]:
@@ -203,8 +236,9 @@ class Hirarchy(object, metaclass=IterRegistry):
 class PropertySet(Hirarchy):
     _registry: set[PropertySet] = set()
 
-    def __init__(self, name: str, obj: Object = None, identifier: str = None) -> None:
-        super(PropertySet, self).__init__(name)
+    def __init__(self, name: str, obj: Object = None, identifier: str = None, description: None | str = None,
+                 optional: None | bool = None) -> None:
+        super(PropertySet, self).__init__(name, description, optional)
         self._attributes = set()
         self._object = obj
         self._registry.add(self)
@@ -213,9 +247,9 @@ class PropertySet(Hirarchy):
             self.identifier = str(uuid4())
         self.changed = True
 
-    def __lt__(self,other):
-        if isinstance(other,PropertySet):
-            return self.name< other.name
+    def __lt__(self, other):
+        if isinstance(other, PropertySet):
+            return self.name < other.name
         else:
             return self.name < other
 
@@ -337,9 +371,10 @@ class Attribute(Hirarchy):
 
     def __init__(self, property_set: PropertySet | None, name: str, value: list, value_type: int,
                  data_type: str = "xs:string",
-                 child_inherits_values: bool = False, identifier: str = None):
+                 child_inherits_values: bool = False, identifier: str = None, description: None | str = None,
+                 optional: None | bool = None):
 
-        super(Attribute, self).__init__(name=name)
+        super(Attribute, self).__init__(name, description, optional)
         self._value = value
         self._property_set = property_set
         self._value_type = value_type
@@ -350,6 +385,7 @@ class Attribute(Hirarchy):
         self.changed = True
         self._child_inherits_values = child_inherits_values
         self.identifier = identifier
+
         if self.identifier is None:
             self.identifier = str(uuid4())
         if property_set is not None:
@@ -360,7 +396,7 @@ class Attribute(Hirarchy):
         return text
 
     def __lt__(self, other):
-        if isinstance(other,Attribute):
+        if isinstance(other, Attribute):
             return self.name < other.name
         else:
             return self.name < other
@@ -477,8 +513,12 @@ class Attribute(Hirarchy):
 
         if equal:
             return True
+        else:
+            return False
 
     def delete(self) -> None:
+        if self in self._registry:
+            self._registry.remove(self)
         self.property_set.remove_attribute(self)
         for child in self.children:
             child.delete()
@@ -500,8 +540,9 @@ class Object(Hirarchy):
     _registry: set[Object] = set()
 
     def __init__(self, name: str, ident_attrib: [Attribute, str], identifier: str = None,
-                 ifc_mapping: set[str] | None = None) -> None:
-        super(Object, self).__init__(name=name)
+                 ifc_mapping: set[str] | None = None, description: None | str = None,
+                 optional: None | bool = None) -> None:
+        super(Object, self).__init__(name, description, optional)
         self._registry.add(self)
 
         self._scripts: list[Script] = list()
@@ -525,10 +566,9 @@ class Object(Hirarchy):
     def __str__(self):
         return f"Object {self.name}"
 
-    def __lt__(self, other:Object):
+    def __lt__(self, other: Object):
         return self.ident_value < other.ident_value
 
-
     @property
     def ifc_mapping(self) -> set[str]:
         return self._ifc_mapping
@@ -664,8 +704,8 @@ class Aggregation(Hirarchy):
     def __str__(self):
         return self.name
 
-    def __init__(self, obj: Object, uuid: str | None = None):
-        super(Aggregation, self).__init__(name=obj.name)
+    def __init__(self, obj: Object, uuid: str | None = None, description: None | str = None,optional: None | bool = None):
+        super(Aggregation, self).__init__(obj.name, description,optional)
         self._registry.add(self)
         if uuid is None:
             self.uuid = str(uuid4())
