@@ -21,8 +21,8 @@ from openpyxl.utils import get_column_letter
 from .. import classes, constants
 from ..Template import IFC_4_1
 
-ident_pset_name = "Allgemeine Eigenschaften"
-ident_attrib_name = "bauteilKlassifikation"
+IDENT_PSET_NAME = "Allgemeine Eigenschaften"
+IDENT_ATTRIB_NAME = "bauteilKlassifikation"
 
 
 def split_string(text: str) -> list[str] | None:
@@ -125,7 +125,7 @@ class ExcelBlock(metaclass=ExcelIterator):
             if abbrev != "-":
                 parent_block = find_by_abbreviation(abbrev)
                 if parent_block is not None:
-                    if parent_block.name != ident_pset_name:
+                    if parent_block.name != IDENT_PSET_NAME:
                         parents.add(parent_block)
                         parents = set.union(parent_block.parent_classes, parents)
                 else:
@@ -162,14 +162,13 @@ class ExcelBlock(metaclass=ExcelIterator):
 
         return inherit_blocks
 
-    def create_attributes(self) -> list[classes.Attribute]:
+    def create_attributes(self) -> set[classes.Attribute]:
         """
            Iterate over Attributes
            Create Them and find special Datatypes
            """
 
-        def transform_value_types(value: str) -> (str, bool):
-            special = False
+        def transform_value_types(attribute_name,value: str) -> (str, bool):
             if value is not None:
                 if value.lower() in ["string", "str"]:
                     data_type = constants.XS_STRING
@@ -180,12 +179,13 @@ class ExcelBlock(metaclass=ExcelIterator):
                 elif value.lower() in ["int", "integer"]:
                     data_type = constants.XS_INT
                 else:
-                    special = True
+                    text = f"{self.name}: Datatype '{value}' of Attribute {attribute_name} can't be interpreted. Use {constants.XS_STRING} instead "
+                    logging.info(text)
                     data_type = constants.XS_STRING
             else:
                 data_type = constants.XS_STRING
 
-            return data_type, special
+            return data_type
 
         if self.is_predefined_pset:
             row = self.base_cell.row + 4
@@ -195,45 +195,58 @@ class ExcelBlock(metaclass=ExcelIterator):
         entry = self.sheet.cell(row=row, column=self.base_cell.column)
 
         cell_list = [block.base_cell for block in ExcelBlock]
-        attributes: list[classes.Attribute] = list()
+        attributes: set[classes.Attribute] = set()
 
         while entry.value is not None and entry not in cell_list and entry.value != "-":
             data_type_text = self.sheet.cell(row=entry.row, column=entry.column + 2).value
-            data_type, special = transform_value_types(data_type_text)
-            attribute_name = entry.value
+            data_type = transform_value_types(entry.value,data_type_text)
+            attribute_name:str = entry.value
             alternative_name = self.sheet.cell(row=entry.row, column=entry.column + 1).value
+
+            optional = False
+            if attribute_name.startswith("*"):
+                optional = True
+                attribute_name = attribute_name[1:]
+
             attribute = classes.Attribute(self.pset, attribute_name, [], constants.VALUE_TYPE_LOOKUP[constants.LIST],
-                                          data_type=data_type)
+                                          data_type=data_type,optional=optional)
             if alternative_name and alternative_name is not None:
                 attribute.revit_name = alternative_name
-            attributes.append(attribute)
-
-            if special:
-                logging.info(
-                    f"[{entry.value}] Property: {self.name}:{attribute_name} datatype '{data_type_text}' unbekannt")
-                pass
+            attributes.add(attribute)
 
             entry = self.sheet.cell(row=entry.row + 1, column=entry.column)
 
         return attributes
 
     def create_predefined_pset(self) -> None:
-        self.pset = classes.PropertySet(self.name)
+        optional = False
+        pset_name = self.name
+        if self.name.startswith("*"):
+            optional = True
+            pset_name = pset_name[1:]
+        self.pset = classes.PropertySet(pset_name,optional=optional)
         self.pset.attributes = self.create_attributes()
 
     def create_object(self) -> classes.Object:
-        self.pset = classes.PropertySet(self.name)
+        pset_name = self.name
+        if self.name.startswith("*"):
+            pset_name = pset_name[1:]
+        self.pset = classes.PropertySet(pset_name)
         self.pset.attributes = self.create_attributes()
         predef_psets: dict[str, ExcelBlock] = {block.name: block for block in ExcelBlock if block.is_predefined_pset}
-        parent_pset = predef_psets.get(ident_pset_name)
+        parent_pset = predef_psets.get(IDENT_PSET_NAME)
 
-        ident_pset = parent_pset.pset.create_child(ident_pset_name)
-        ident_attrib = ident_pset.get_attribute_by_name(ident_attrib_name)
+        ident_pset = parent_pset.pset.create_child(IDENT_PSET_NAME)
+        ident_attrib = ident_pset.get_attribute_by_name(IDENT_ATTRIB_NAME)
         ident_attrib.value = [self.ident_value]
-        obj = classes.Object(self.name, ident_attrib)
+        optional = False
+        obj_name = self.name
+        if self.name.startswith("*"):
+            optional = True
+            obj_name= obj_name[1:]
+        obj = classes.Object(obj_name, ident_attrib,ifc_mapping=self.ifc_mapping(),optional=optional)
         obj.add_property_set(self.pset)
         obj.add_property_set(ident_pset)
-        obj.ifc_mapping = self.ifc_mapping()
         obj.custom_attribues[constants.ABBREVIATION] = self.abbreviation
 
         return obj
@@ -290,6 +303,14 @@ def _create_items() -> None:
             new_pset = pset.create_child(pset.name)
             block.object.add_property_set(new_pset)
 
+    #for SOM of Deutsche Bahn
+    for obj in classes.Object:
+        prop = obj.get_property_set_by_name("Allgemeine Eigenschaften")
+        if prop is None:
+            continue
+        bn = prop.get_attribute_by_name("bauteilName")
+        if bn is not None:
+            bn.value = [obj.name]
 
 def _build_object_tree() -> None:
     tree_dict: dict[str, classes.Object] = {obj.ident_attrib.value[0]: obj for obj in classes.Object}
@@ -398,9 +419,10 @@ def export(project: classes.Project, path: str, mapping_dict: dict[str, str] = {
     if not os.path.exists(os.path.dirname(path)):
         raise FileNotFoundError(f"path {os.path.dirname(path)} DNE")
 
-    NA  = "name"
-    OBJ = "objects"
+    NAME = "name"
+    OBJECTS = "objects"
     TABLE_STYLE = "TableStyleLight1"
+    OPTIONAL_FONT = styles.Font(color="4e6ec0")
     def fill_main_sheet(sheet: Worksheet) -> None:
         sheet.title = "Uebersicht"
         sheet.cell(1, 1).value = "bauteilName"
@@ -409,7 +431,9 @@ def export(project: classes.Project, path: str, mapping_dict: dict[str, str] = {
         for row, obj in enumerate(sorted(project.objects), start=2):
             sheet.cell(row, 1).value = obj.name
             sheet.cell(row, 2).value = str(obj.ident_value)
-
+            if obj.optional:
+                sheet.cell(row,1).font = OPTIONAL_FONT
+                sheet.cell(row,2).font = OPTIONAL_FONT
         table_start = sheet.cell(1,1).coordinate
         table_end = sheet.cell(row,2).coordinate
         table_range = f"{table_start}:{table_end}"
@@ -421,16 +445,28 @@ def export(project: classes.Project, path: str, mapping_dict: dict[str, str] = {
         autoadjust_column_widths(sheet)
 
     def filter_to_sheets() -> dict:
-        d = {obj.ident_value: {NA:obj.name,OBJ:[]} for obj in project.objects if len(obj.ident_value.split(".")) == 1}
+        d = {obj.ident_value: {NAME:obj.name,OBJECTS:[]} for obj in project.objects if len(obj.ident_value.split(".")) == 1}
         for ident,name in mapping_dict.items():
-            d[ident] = {NA:name,OBJ:[]}
+            d[ident] = {NAME:name,OBJECTS:[]}
 
         for obj in project.objects:
             group = obj.ident_value.split(".")[0]
-            d[group][OBJ].append(obj)
+            d[group][OBJECTS].append(obj)
+
+        d["son"]= {NAME:"Sonstige",OBJECTS:[]}
+        for group_name,group in list(d.items()):
+            objects = group[OBJECTS]
+            if len(objects)<3:
+                d["son"][OBJECTS]+=objects
+                del d[group_name]
         return d
 
     def create_object_entry(obj:classes.Object, sheet, start_row, start_column,table_index):
+        if obj.optional:
+            font_style = OPTIONAL_FONT
+        else:
+            font_style = styles.Font()
+
         sheet.cell(start_row,start_column).value = "bauteilName"
         sheet.cell(start_row,start_column+1).value = obj.name
 
@@ -444,6 +480,9 @@ def export(project: classes.Project, path: str, mapping_dict: dict[str, str] = {
         sheet.cell(start_row+3,start_column+1).value = "Propertyset"
         sheet.cell(start_row+3,start_column+2).value = "Wertebereich"
 
+        for i in range(1,4):
+            for k in range(1,3):
+                sheet.cell(start_row+i,start_column+k).font = font_style
         draw_border(sheet, [start_row, start_row+2], [start_column, start_column+2])
         fill_grey(sheet, [start_row, start_row+2], [start_column, start_column+2])
 
@@ -459,6 +498,10 @@ def export(project: classes.Project, path: str, mapping_dict: dict[str, str] = {
                 else:
                     attrib_val = ";".join(attribute.value)
                 sheet.cell(pset_start_row+index,start_column+2).value = attrib_val
+                if attribute.optional:
+                    sheet.cell(pset_start_row + index, start_column).font = OPTIONAL_FONT
+                    sheet.cell(pset_start_row + index, start_column+1).font = OPTIONAL_FONT
+                    sheet.cell(pset_start_row + index, start_column+2).font = OPTIONAL_FONT
                 index+=1
 
         table_start = sheet.cell(pset_start_row-1,start_column).coordinate
@@ -503,8 +546,8 @@ def export(project: classes.Project, path: str, mapping_dict: dict[str, str] = {
     sheet_dict = filter_to_sheets()
     table_counter = 1
     for ident,d in sheet_dict.items():
-        obj_name = d[NA]
-        objects = d[OBJ]
+        obj_name = d[NAME]
+        objects = d[OBJECTS]
         work_sheet = workbook.create_sheet(f"{obj_name} ({ident})")
         for counter,obj in enumerate(sorted(objects)):
             column = 1+counter*4
